<?php
namespace grUPN;

/** 
 * Plugin Name: GR UPN plačilni nalog
 * Description: Vtičnik omogoča generiranje UPN plačilnega naloga s QR kodo in pošiljanje stranki po e-pošti. Brezplačna različica omogoča generiranje in pošiljanje UPN naloga za posamezna naročila. Nadgradite na Pro različico za avtomatsko generiranje UPN obrazca, ki se prikaže ob zaključku naročila poleg podrobnosti za najlažje direktno plačevanje.
 * Version: 1.0
 * Author: Grega Rednak
 * Author URI: https://gregarednak.com
 * Tested up to: 6.4.2
 * Tags: woocommerce, UPN, woocommerce payment, payment, order pay qr, payment qr, qr, plačila, plačilo, plačilni nalog
 * License: GPL-3.0
 * License URI: https://www.gnu.org/licenses/gpl-3.0.html
 */

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

if (!function_exists('grfreemium')) {
    // Create a helper function for easy SDK access.
    function grfreemium()
    {
        global $grfreemium;

        if (!isset($grfreemium)) {
            // Include Freemius SDK.
            require_once dirname(__FILE__) . '/freemius/start.php';

            $grfreemium = fs_dynamic_init(
                array(
                    'id' => '14756',
                    'slug' => 'gr-upn',
                    'type' => 'plugin',
                    'public_key' => 'pk_82748159feee520dfb4bf6fec3797',
                    'is_premium' => true,
                    'premium_suffix' => 'Pro',
                    // If your plugin is a serviceware, set this option to false.
                    'has_premium_version' => true,
                    'has_addons' => false,
                    'has_paid_plans' => true,
                    'is_org_compliant' => false,
                    'menu' => array(
                        'first-path' => 'plugins.php',
                        'contact' => false,
                        'support' => false,
                    ),
                )
            );
        }

        return $grfreemium;
    }

    // Init Freemius.
    grfreemium();
    // Signal that SDK was initiated.
    do_action('grfreemium_loaded');
}

require_once(plugin_dir_path(__FILE__) . 'vendor/autoload.php');

grfreemium()->add_action('after_uninstall', 'grUPN\\uninstall_cleanup');
function uninstall_cleanup()
{
    delete_option('gr_namen_placila');
    delete_option('gr_koda_namena');
    delete_option('gr_iban_prejemnika');
    delete_option('gr_referenca_prejemnika');
    delete_option('gr_ime_prejemnika');
    delete_option('gr_ulica_prejemnika');
    delete_option('gr_kraj_prejemnika');
    delete_option('gr_posta_prejemnika');
    delete_option('gr_email_attachment');
    delete_option('gr_email_insert_upn');
}

add_action('woocommerce_init', 'grUPN\\grUPN_init_on_checkout', 5, 3);
function grUPN_init_on_checkout()
{
    new LogicUPN();
}


/* FREE functions */
add_action('wp_ajax_send_upn_to_customer', 'grUPN\\gr_upn_ajax_send_upn_to_customer');
function gr_upn_ajax_send_upn_to_customer()
{
    $order_id = isset($_POST['order_id']) ? intval($_POST['order_id']) : 0;
    if ($order_id > 0) {
        $order = wc_get_order($order_id);
        if ($order && current_user_can('edit_shop_orders')) {
            gr_upn_send_upn_email($order_id);
            wp_send_json_success('UPN uspešno poslan.');
        }
    }

    wp_send_json_error('Napaka pri pošiljanju UPN.');
    wp_die();
}

function gr_upn_send_upn_email($order_id)
{
    $order = wc_get_order($order_id);
    if ('bacs' === $order->get_payment_method()) {
        $UPN = new LogicUPN();
        $png = $UPN->genUPN($order);
        $upn_image_path = $UPN->save_upn_image($png, $order->get_id());

        if ($upn_image_path) {
            // Prepare the email parts
            $subject = 'UPN obrazec za naročilo #' . $order_id;
            $message = 'V prilogi imate UPN obrazec za lažje plačilo na bančni račun.';
            $headers = array('Content-Type: text/html; charset=UTF-8');

            // Assuming the email of the order's customer is the recipient
            $recipient = $order->get_billing_email();

            // Attachments should be a file path
            $attachments = array($upn_image_path);

            // Send the email
            wp_mail($recipient, $subject, $message, $headers, $attachments);

            // Clean up: Delete the image file after sending the email
            if (file_exists($upn_image_path)) {
                unlink($upn_image_path);
            }
        }
    }
}
/* End of FREE functions */

if (is_admin()) {
    add_action("admin_init", function () {
        if (wp_doing_ajax()) {
            // Skip admin notices if it's an AJAX request
            return;
        }
        if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
            ?>
            <div class="notice notice-error">
                <p>
                    <?php echo 'GR UPN plačilni nalog - QR vtičnik potrebuje <a href="http://wordpress.org/extend/plugins/woocommerce/">woocommerce</a> vtičnik za delovanje!'; ?>
                </p>
            </div>
            <?php
        }
        if (grfreemium()->is_not_paying()) {
            ?>
            <div class="notice notice-warning is-dismissible">
                <p>
                    <?php echo '<a href="' . grfreemium()->get_upgrade_url() . '">Nagradite</a> za avtomatsko generiranje in prikazovanje UPN obrazcev.'; ?>
                </p>
            </div>
            <?php
        }
    });

    function grUPN_add_upn_settings($settings)
    {
        if (class_exists('WC_Settings_Page')) {
            $settings[] = new WC_Settings_UPN();
        }
        return $settings;
    }
    add_filter('woocommerce_get_settings_pages', 'grUPN\\grUPN_add_upn_settings');


    function gr_upn_enqueue_scripts($hook)
    {
        // Register the style
        wp_register_style('main_css', plugin_dir_url(__FILE__) . 'src/assets/css/main.css');

        // Enqueue the style
        wp_enqueue_style('main_css');

        // Enqueue the js
        wp_enqueue_script(
            'gr_upn_admin_script',
            plugin_dir_url(__FILE__) . 'src/assets/js/admin.js',
            array('jquery'),
            '1.0.0',
            true
        );
        wp_localize_script('gr_upn_admin_script', 'grUPNAjax', array('ajaxurl' => admin_url('admin-ajax.php')));
    }
    // Enqueue scripts and styles for the admin pages
    add_action('admin_enqueue_scripts', 'grUPN\\gr_upn_enqueue_scripts');

    // Corrected plugin action links
    $plugin = plugin_basename(__FILE__);
    add_filter("plugin_action_links_$plugin", 'grUPN\\add_plugin_settings_link');
    function add_plugin_settings_link($links)
    {
        $links[] = '<a href="admin.php?page=wc-settings&tab=upn_settings">Nastavitve</a>';
        $links[] = '<a href="' . grfreemium()->get_upgrade_url() . '">Nagradite</a>';
        return $links;
    }

    // Corrected plugin row meta
    add_filter('plugin_row_meta', 'grUPN\\add_plugin_row_meta', 10, 2);
    function add_plugin_row_meta($plugin_meta, $plugin_file)
    {
        if (strpos($plugin_file, 'gr-upn.php') !== false) {
            $plugin_meta[] = '<a href="admin.php?page=wc-settings&tab=upn_settings">Nastavitve</a>';
            $plugin_meta[] = '<a href="' . grfreemium()->get_upgrade_url() . '">Nagradite</a>';
        }
        return $plugin_meta;
    }
}


if (!wp_next_scheduled('gr_upn_daily_cleanup')) {
    wp_schedule_event(time(), 'daily', 'gr_upn_daily_cleanup');
}
add_action('gr_upn_daily_cleanup', 'gr_upn_cleanup_images');
function gr_upn_cleanup_images()
{
    $upload_dir = wp_upload_dir();
    $dir = $upload_dir['path'];
    $files = glob($dir . '/gr-upn-order-*.png'); // Adjust the pattern to match your filenames

    foreach ($files as $file) {
        if (is_file($file)) {
            // Check if the file is at least 1 day old
            if (time() - filemtime($file) > 86400) { // 86400 seconds = 1 day
                unlink($file);
            }
        }
    }
}