<?php
namespace grUPN;

class LogicUPN
{

  public $bacs_account;
  public function __construct()
  {
    // Get the gateways instance
    $gateways = \WC_Payment_Gateways::instance();

    // Get all available gateways, [id] => Object
    $available_gateways = $gateways->get_available_payment_gateways();
    if (isset($available_gateways['bacs'])) {
      if (!grfreemium()->is_not_paying()) {
        add_action('woocommerce_email_after_order_table', array($this, 'displayUPNOnEmails'), 10, 3);
        add_action('woocommerce_thankyou_bacs', array($this, 'displayUPNOnOrderPage'), 10, 3);
      }
      $bacs_accounts = $available_gateways['bacs']->account_details;

      $this->bacs_account = (object) $bacs_accounts[0];

      add_action('woocommerce_order_item_add_action_buttons', array($this, 'gr_upn_add_send_upn_button'), 10, 3);
    }
  }

  public function gr_upn_add_send_upn_button($order)
  {
    if ('bacs' === $order->get_payment_method()) {
      echo '<button type="button" class="button" data-order-id="' . $order->get_id() . '" id="gr-generate-and-send-upn-button">Pošlji UPN stranki</button>';
    }
  }

  public function genUPN($order)
  {
    $png = (new UPN())
      ->setPayerName(sprintf("%s", $order->get_formatted_billing_full_name()))
      ->setPayerAddress($order->get_billing_address_1())
      ->setPayerPost(sprintf("%s %s", $order->get_billing_postcode(), $order->get_billing_city()))
      ->setReceiverName(get_option('gr_ime_prejemnika', $this->bacs_account->account_name))
      ->setReceiverAddress(get_option('gr_ulica_prejemnika', WC()->countries->get_base_address()))
      ->setReceiverPost(sprintf("%s %s", get_option('gr_posta_prejemnika', WC()->countries->get_base_postcode()), get_option('gr_kraj_prejemnika', WC()->countries->get_base_city())))
      ->setReceiverIban(get_option('gr_iban_prejemnika', $this->bacs_account->iban))
      ->setAmount($order->get_total())
      ->setCode(get_option('gr_koda_namena', $this->bacs_account->sort_code))
      ->setReference(sprintf("%s %s", get_option('gr_referenca_prejemnika', 'SI00'), $order->get_order_number()))
      ->setDueDate(new \DateTime($order->get_date_created()))
      ->setPurpose(sprintf("%s %s", get_option('gr_namen_placila', 'Plačilo naročila '), $order->get_order_number()))
      ->png();

    // Check for gd errors / buffer errors
    if (!empty($png)) {
      return $png;
    }
  }

  public function displayUPNOnOrderPage($order_id)
  {
    // Display UPN on the order received page
    if (!$order_id) {
      return;
    }

    // Get the order object
    $order = wc_get_order($order_id);

    // Check if the order payment method is BACS
    if ($order->get_payment_method() === 'bacs') {
      $png = $this->genUPN($order);
      // Display the image
      if ($png) {
        echo '<div class="gr-upn">';
        echo '<img src="data:image/png;base64,' . base64_encode($png) . '" alt="UPN QR Code">';
        echo '</div>';
      }
    }
  }

  public function displayUPNOnEmails($order, $sent_to_admin, $plain_text)
  {
    if (!$sent_to_admin && 'bacs' === $order->get_payment_method()) {
      $png = $this->genUPN($order);
      $upn_image_url = $this->save_upn_image($png, $order->get_id());

      if ($upn_image_url && get_option('gr_email_priponka')) {
        add_filter('woocommerce_email_attachments', function ($attachments, $object) use ($upn_image_url, $order) {
          if (!empty($upn_image_url)) {
            // Add the image to the attachments array
            $attachments[] = wp_upload_dir()['path'] . '/gr-upn-order-' . $order->get_id() . '.png';
            return $attachments;
          }
        }, 10, 2);
      }

      if ($upn_image_url && get_option('gr_email_vstavi_upn')) {
        echo '<div class="gr-upn" style="margin-top: 20px;">';
        if (get_option('gr_email_priponka')) {
          $file_name = 'gr-upn-order-' . $order->get_id() . '.png';
          echo '<img src="cid:' . $file_name . '" alt="Univerzalni plačilni nalog (UPN) za plačilo naročila">';
        } else {
          echo '<img src="' . esc_url($upn_image_url) . '" alt="UPN - univerzalni plačilni nalog za plačilo naročila">';
        }
        echo '</div>';
      }
    }
  }


  public function save_upn_image($img, $order_id)
  {
    $file_path = wp_upload_dir()['path'] . '/gr-upn-order-' . $order_id . '.png';

    // Save the image
    file_put_contents($file_path, $img);

    // Check if the site is using SSL and modify the URL accordingly
    $upn_image_url = wp_upload_dir()['path'] . '/gr-upn-order-' . $order_id . '.png';
    if (is_ssl()) {
      $upn_image_url = str_replace('http://', 'https://', $upn_image_url);
    }

    return $upn_image_url;
  }


}