<?php
namespace grUPN;

use InvalidArgumentException;

class UPN
{
  private string $ime_placnika;
  private string $naslov_placnika;
  private string $mesto_placnika;

  private string $ime_prejemnika;
  private string $naslov_prejemnika;
  private string $mesto_prejemnika;
  private string $receiver_iban;
  private string $reference;

  private float $amount;
  private string $koda_namena;
  private string $namen_placila = '';
  private \DateTime $due_date;

  private const FONT = __DIR__ . '/assets/fonts/courier-new-bold.ttf';
  private const FONT_SIZE = 17;
  private const FONT_SMALL = 12;

  private $image;
  private $color;

  public function __construct()
  {
    $this->image = imagecreatefrompng(__DIR__ . '/assets/media/upn.png');
    $this->color = imagecolorallocate($this->image, 0x00, 0x00, 0x00);
  }

  public function gdResource()
  {
    $this->writeText(697, 170, $this->ime_placnika ?? '');
    $this->writeText(697, 201, $this->naslov_placnika ?? '');
    $this->writeText(697, 233, $this->mesto_placnika ?? '');

    $this->writeText(30, 62, $this->ime_placnika ?? '', self::FONT_SMALL);
    $this->writeText(30, 87, $this->naslov_placnika ?? '', self::FONT_SMALL);
    $this->writeText(30, 112, $this->mesto_placnika ?? '', self::FONT_SMALL);

    $this->writeText(418, 507, $this->ime_prejemnika ?? '');
    $this->writeText(418, 538, $this->naslov_prejemnika ?? '');
    $this->writeText(418, 570, $this->mesto_prejemnika ?? '');

    $this->writeText(30, 405, $this->ime_prejemnika ?? '', self::FONT_SMALL);
    $this->writeText(30, 430, $this->naslov_prejemnika ?? '', self::FONT_SMALL);
    $this->writeText(30, 455, $this->mesto_prejemnika ?? '', self::FONT_SMALL);

    $this->writeText(418, 400, $this->getFormatedReceiverIban() ?? '');
    $this->writeText(30, 300, $this->getFormatedReceiverIban() ?? '', self::FONT_SMALL);

    $this->writeText(418, 451, $this->getReferencePrefix());
    $this->writeText(528, 451, $this->getReferenceSuffix());
    $this->writeText(30, 351, $this->getFormatedReference(), self::FONT_SMALL);

    $this->writeText(528, 340, $this->namen_placila ?? '');
    $this->writeText(30, 165, $this->namen_placila ?? '', 10);

    $this->writeText(1155, 340, $this->due_date->format('d.m.Y'));
    $this->writeText(30, 195, $this->due_date->format('d.m.Y'), self::FONT_SMALL);

    $this->writeText(110, 247, '***' . $this->getFormatedPrice(), self::FONT_SMALL);
    $this->writeText(750, 285, '***' . $this->getFormatedPrice());

    $this->writeText(418, 340, $this->koda_namena ?? '');

    $qr = $this->getQRCode();
    imagecopyresampled($this->image, $qr, 433, 60, 0, 0, 220, 220, imagesx($qr), imagesy($qr));

    return $this->image;
  }

  public function png(): string
  {
    $image = $this->gdResource();

    ob_start();
    imagepng($image);
    $img = ob_get_contents();
    ob_end_clean();

    imagedestroy($image);

    return $img;
  }

  public function getQRCode()
  {

    $original_error_reporting = error_reporting();
    error_reporting(E_ERROR | E_PARSE);
    $qrEncoder = \QR_Code\Encoder\Encoder::factory(QR_ECLEVEL_M, 3, 0);
    $qrEncoder->version = 15;

    $tab = $qrEncoder->encode($this->getQRCodeText());

    $image = \QR_Code\Encoder\Image::image(
      $tab,
      min(max(1, 3), (int) (QR_PNG_MAXIMUM_SIZE / (count($tab)))),
      0
    );

    error_reporting($original_error_reporting);
    return $image
    ;

  }

  public function getQRCodeText(): string
  {
    $text = [
      'UPNQR',
      '',
      '    ',
      '',
      '',
      $this->ime_placnika ?? '',
      $this->naslov_placnika ?? '',
      $this->mesto_placnika ?? '',
      sprintf('%011d', $this->amount * 100) ?? '',
      '',
      '',
      $this->koda_namena ?? '',
      $this->namen_placila ?? '',
      $this->due_date->format('d.m.Y') ?? '',
      $this->receiver_iban ?? '',
      $this->reference ?? '',
      $this->ime_prejemnika ?? '',
      $this->naslov_prejemnika ?? '',
      $this->mesto_prejemnika ?? '',
    ];

    array_walk($text, fn(&$i) => $i = trim($i));
    $text = implode("\n", $text) . "\n";
    $text .= mb_strlen($text) . "\n"; // append control code

    return $text;
  }

  public function getFormatedPrice(): string
  {
    return number_format($this->amount, 2, ',', '.');
  }

  public function getFormatedReceiverIban(): string
  {
    return wordwrap($this->receiver_iban, 4, ' ', true);
  }

  public function getFormatedReference(): string
  {
    return $this->getReferencePrefix() . ' ' . $this->getReferenceSuffix();
  }

  public function getReferencePrefix(): string
  {
    return substr($this->reference, 0, 4);
  }

  public function getReferenceSuffix(): string
  {
    return substr($this->reference, 4);
  }

  private function writeText(int $x, int $y, $text, $fontSize = self::FONT_SIZE): void
  {
    if ($text) {
      imagefttext($this->image, $fontSize, 0, $x, $y, $this->color, self::FONT, $text);
    }
  }

  public function getPayerName(): string
  {
    return $this->ime_placnika;
  }

  public function setPayerName(string $ime_placnika): self
  {
    $this->ime_placnika = mb_substr($ime_placnika, 0, 33);

    return $this;
  }

  public function getPayerAddress(): string
  {
    return $this->naslov_placnika;
  }

  public function setPayerAddress(string $naslov_placnika): self
  {
    $this->naslov_placnika = mb_substr($naslov_placnika, 0, 33);

    return $this;
  }

  public function getPayerPost(): string
  {
    return $this->mesto_placnika;
  }

  public function setPayerPost(string $mesto_placnika): self
  {
    $this->mesto_placnika = mb_substr($mesto_placnika, 0, 33);

    return $this;
  }

  public function getReceiverName(): string
  {
    return $this->ime_prejemnika;
  }

  public function setReceiverName(string $ime_prejemnika): self
  {
    $this->ime_prejemnika = mb_substr($ime_prejemnika, 0, 33);

    return $this;
  }

  public function getReceiverAddress(): string
  {
    return $this->naslov_prejemnika;
  }

  public function setReceiverAddress(string $naslov_prejemnika): self
  {
    $this->naslov_prejemnika = mb_substr($naslov_prejemnika, 0, 33);

    return $this;
  }

  public function getReceiverPost(): string
  {
    return $this->mesto_prejemnika;
  }

  public function setReceiverPost(string $mesto_prejemnika): self
  {
    $this->mesto_prejemnika = mb_substr($mesto_prejemnika, 0, 33);

    return $this;
  }

  public function getReceiverIban(): string
  {
    return $this->receiver_iban;
  }

  public function setReceiverIban(string $receiver_iban): self
  {
    $iban = str_replace(' ', '', $receiver_iban);

    if (strlen($iban) > 33) {
      throw new InvalidArgumentException('Najdaljši IBAN je 33 znakov');
    }

    $this->receiver_iban = $iban;

    return $this;
  }

  public function getReference(): string
  {
    return $this->reference;
  }

  public function setReference(string $reference): self
  {
    $reference = str_replace(' ', '', $reference);

    if (strlen($reference) > 26) {
      throw new InvalidArgumentException('Najdaljša referenca je 26 znakov');
    }

    $this->reference = $reference;

    return $this;
  }

  public function getAmount(): float
  {
    return $this->amount;
  }

  public function setAmount(float $amount): self
  {
    $this->amount = $amount;

    return $this;
  }

  public function getCode(): string
  {
    return $this->koda_namena;
  }

  public function setCode(string $koda_namena): self
  {
    if (strlen($koda_namena) !== 4) {
      throw new InvalidArgumentException('Koda namena mora biti 4 črke');
    }
    $this->koda_namena = strtoupper($koda_namena);

    return $this;
  }

  public function getPurpose(): string
  {
    return $this->namen_placila;
  }

  public function setPurpose(string $namen_placila): self
  {
    $this->namen_placila = mb_substr($namen_placila, 0, 42);

    return $this;
  }

  public function getDueDate(): \DateTime
  {
    return $this->due_date;
  }

  public function setDueDate(\DateTime $due_date): self
  {
    $this->due_date = $due_date;

    return $this;
  }
}