<?php
namespace grUPN;

class WC_Settings_UPN extends \WC_Settings_Page
{

    public function __construct()
    {
        $this->id = 'upn_settings';
        $this->label = __('UPN Nastavitve', 'woocommerce');

        add_filter('woocommerce_settings_tabs_array', array($this, 'add_settings_tab'), 50);
        add_action('woocommerce_settings_' . $this->id, array($this, 'output'));
        add_action('woocommerce_settings_save_' . $this->id, array($this, 'save'));
    }
    public function add_settings_tab($tabs)
    {
        $tabs[$this->id] = $this->label;
        return $tabs;
    }

    public function get_settings()
    {
        if (grfreemium()->is_not_paying()) {
            ?>
            <div class="notice notice-warning">
                <p>
                    <?php echo 'Imate omejitev generiranja UPN obrazcev na: ' . 25 . '. <a href="' . grfreemium()->get_upgrade_url() . '">Nagradite</a> za neomejeno uporabo.'; ?>
                </p>
            </div>
            <?php
        }

        $available_gateways = \WC_Payment_Gateways::instance()->get_available_payment_gateways();
        if (!isset($available_gateways['bacs'])) {
            add_action('admin_notices', function () {
                echo '<div class="notice notice-error"><p>Direct bank transfer payment option not set in WooCommerce.</p></div>';
            });
            return [];
        }
        $bacs_accounts = $available_gateways['bacs']->account_details;

        $bacs_account = (object) $bacs_accounts[0];
        $settings = array(
            array(
                'title' => __('UPN Nastavitve', 'woocommerce'),
                'type' => 'title',
                'desc' => __('Nastavitve za UPN QR plačila.', 'woocommerce'),
                'id' => 'upn_settings_options'
            ),
            array(
                'title' => __('Namen plačila', 'woocommerce'),
                'id' => 'gr_namen_placila',
                'type' => 'text',
                'default' => 'Plačilo naročila',
                'desc_tip' => true,
            ),
            array(
                'title' => __('Koda namena', 'woocommerce'),
                'id' => 'gr_koda_namena',
                'type' => 'text',
                'default' => $bacs_account->sort_code,
                'desc_tip' => true,
            ),
            array(
                'title' => __('IBAN prejemnika', 'woocommerce'),
                'id' => 'gr_iban_prejemnika',
                'type' => 'text',
                'default' => $bacs_account->iban,
                'desc_tip' => true,
            ),
            array(
                'title' => __('Referenca prejemnika', 'woocommerce'),
                'id' => 'gr_referenca_prejemnika',
                'type' => 'text',
                'default' => 'SI00',
                'desc_tip' => true,
            ),
            array(
                'title' => __('Ime prejemnika', 'woocommerce'),
                'id' => 'gr_ime_prejemnika',
                'type' => 'text',
                'default' => $bacs_account->account_name,
                'desc_tip' => true,
            ),
            array(
                'title' => __('Ulica prejemnika', 'woocommerce'),
                'id' => 'gr_ulica_prejemnika',
                'type' => 'text',
                'default' => WC()->countries->get_base_address(),
                'desc_tip' => true,
            ),
            array(
                'title' => __('Kraj prejemnika', 'woocommerce'),
                'id' => 'gr_kraj_prejemnika',
                'type' => 'text',
                'default' => WC()->countries->get_base_city(),
                'desc_tip' => true,
            ),
            array(
                'title' => __('Poštna številka prejemnika', 'woocommerce'),
                'id' => 'gr_posta_prejemnika',
                'type' => 'text',
                'default' => WC()->countries->get_base_postcode(),
                'desc_tip' => true,
            ),
            array(
                'title' => __('Dodaj UPN v email priponko (PRO)', 'woocommerce'),
                'id' => 'gr_email_priponka',
                'type' => 'checkbox',
                'default' => true,
                'desc_tip' => true,
                'disabled' => grfreemium()->is_not_paying() ? 'disabled' : '', // Disable if free version
            ),
            array(
                'title' => __('Dodaj UPN kot sliko v email sporočilo (PRO)', 'woocommerce'),
                'id' => 'gr_email_vstavi_upn',
                'type' => 'checkbox',
                'default' => true,
                'desc_tip' => true,
                'desc' => __('Če email blokira vaše slike v sporočilu poskusite dodati UPN tudi v priponko', 'woocommerce'),
                'disabled' => grfreemium()->is_not_paying() ? 'disabled' : '', // Disable if free version
            ),
            array(
                'type' => 'sectionend',
                'id' => 'upn_settings_options'
            ),
        );

        return apply_filters('woocommerce_get_settings_' . $this->id, $settings);
    }
}